
const React = require('react')
const ReactDOMServer = require('react-dom/server')

const HTMLView = props => null
HTMLView['Content-Type'] = 'text/html'
HTMLView.is_compressable = true
HTMLView.makeEmit = ({ descriptor }) => (
    React.createElement(descriptor.props.component, descriptor.gathered_props)
)
HTMLView.doctype = '<!doctype html>'
HTMLView.default_props = {
    fragment: false
}

const SitemapView = props => null
SitemapView['Content-Type'] = 'text/plain'
SitemapView.is_compressable = true
SitemapView.default_props = {
    path: 'sitemap.txt'
}
SitemapView.doctype = false
SitemapView.makeEmit = ({ descriptor, all_descriptors, config }) => {

    const links = new Set()
    all_descriptors.forEach( d => {
        if (null != d.evaluated_path) {
            links.add(( config.HTTPS ? 'https' : 'http' ) + '://' + config.HOST + d.evaluated_path)
        }
    })

    return [...links.values()].join('\n')
}




const RSSView = props => null
RSSView['Content-Type'] = 'application/rss+xml'
RSSView.is_compressable = true
RSSView.makeEmit = ({ descriptor }) => (
    React.createElement(descriptor.props.component, descriptor.gathered_props)
)
RSSView.doctype = '<?xml version="1.0"?>'

const Enumerate = props => null
Enumerate.Log = props => null

const AssetPipeline = props => null

const Skip = props => null

AssetPipeline.Skip  = Skip
Enumerate.Skip      = Skip
HTMLView.Skip       = Skip
RSSView.Skip        = Skip
SitemapView.Skip    = Skip

module.exports = {
    AssetPipeline,
    Enumerate,
    HTMLView,
    RSSView,
    SitemapView,
    Skip,
}